from dataclasses import dataclass
from datetime import datetime
from typing import List

from torrent_tracker_search.common.model.ISearchResultItem import ISearchResultItem
from torrent_tracker_search.source.rutracker.model.RuTrackerAuthorLink import RuTrackerAuthorLink
from torrent_tracker_search.source.rutracker.model.RuTrackerForumLink import RuTrackerForumLink
from torrent_tracker_search.source.rutracker.model.RuTrackerTopicLink import RuTrackerTopicLink


@dataclass
class RuTrackerSearchItem(ISearchResultItem):
	id: int  # id for download torrent file
	is_verified: bool
	topic: RuTrackerTopicLink
	author: RuTrackerAuthorLink
	forum: RuTrackerForumLink
	tags: List[str]
	size: int
	seed_count: int
	leech_count: int
	download_count: int
	added_at: datetime
	
	@staticmethod
	def empty(id: int):
		return RuTrackerSearchItem(
			id=id,
			is_verified=False,
			topic=RuTrackerTopicLink(0, ""),
			author=RuTrackerAuthorLink(0, ""),
			forum=RuTrackerForumLink(0, ""),
			tags=[],
			size=0,
			seed_count=0,
			leech_count=0,
			download_count=0,
			added_at=datetime.now()
		)
	


from typing import List, Optional, Final, Dict

from aiohttp import ClientSession

from torrent_tracker_search.common.TrackerClient import TrackerClient


class ClientBuilder:
	
	DEFAULT_HEADERS: Final[Dict[str, str]] = {
		"User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36"
	}
	
	def __init__(self, session: Optional[ClientSession] = None):
		self.session: ClientSession = session or ClientSession(headers=self.DEFAULT_HEADERS)
	
	def build(self, clients: List[TrackerClient]) -> List[TrackerClient]:
		for client in clients:
			client.session = self.session
		return clients



from enum import Enum

from torrent_tracker_search.source.nnmclub.NNMClubClient import NNMClubClient


class NNMClubURL(Enum):
	MAIN = "index.php"
	TRACKER = "tracker.php"
	POSTING = "posting.php"
	VIEW_TOPIC = "viewtopic.php"
	FILE_LIST = "filelst.php"
	
	@property
	def url(self) -> str:
		return f"{NNMClubClient.BASE_URL}/{self.value}"



from typing import Final, Optional

from torrent_tracker_search.common.TrackerClient import TrackerClient
from torrent_tracker_search.common.TrackerRequestException import TrackerRequestException
from torrent_tracker_search.common.TrackerResponse import TrackerResponse


class NNMClubClient(TrackerClient):
	BASE_URL: Final[str] = "https://nnmclub.to/forum"
	SESSION_COOKIE_KEY: Final[str] = "phpbb2mysql_4_sid"
	
	async def get(self, url: str, params: Optional[dict] = None, allow_redirects: bool = True) -> TrackerResponse:
		async with self.session.get(url=url, params=params, ssl=False, allow_redirects=allow_redirects) as response:
			success_status_code = 200 if allow_redirects else 302
			if response.status != success_status_code:
				raise TrackerRequestException()
			else:
				return TrackerResponse(
					headers=response.headers.copy(),
					status_code=response.status,
					body=await response.content.read()
				)
	
	async def post_form_data_encoded(self, url: str, body: dict, params: Optional[dict] = None, allow_redirects: bool = True) -> TrackerResponse:
		async with self.session.post(url=url, data=body, ssl=False, allow_redirects=allow_redirects, params=params) as response:
			success_status_code = 200 if allow_redirects else 302
			if response.status == success_status_code:
				return TrackerResponse(
					headers=response.headers.copy(),
					status_code=response.status,
					body=await response.content.read()
				)
			else:
				raise TrackerRequestException()
			
	async def post_form_data(self, url: str, body: dict, params: Optional[dict] = None, allow_redirects: bool = True) -> TrackerResponse:
		async with self.session.post(url=url, data=body, ssl=False, allow_redirects=allow_redirects, params=params) as response:
			success_status_code = 200 if allow_redirects else 302
			if response.status == success_status_code:
				return TrackerResponse(
					headers=response.headers.copy(),
					status_code=response.status,
					body=await response.content.read()
				)
			else:
				raise TrackerRequestException()
			


from typing import Optional

from bs4 import BeautifulSoup, Tag

from torrent_tracker_search.source.nnmclub.parser import get_torrent_id


class NNMClubTorrentParser:
	
	@classmethod
	def parse(cls, html: str) -> Optional[dict]:
		soup = BeautifulSoup(html, "html.parser")
		table = soup.find("th", string="Сообщение").find_parent("table")
		tables = [tag for tag in table.childGenerator() if type(tag) is Tag]
		content_table = [tag for tag in tables[1].childGenerator() if type(tag) is Tag][1].table
		if content_table is not None:
			items = [tag for tag in content_table.childGenerator() if type(tag) is Tag]
			header, _, torrent_page, torrent_details, _ = items
			magnet_link = torrent_details.find("a", attrs={"title": "Примагнититься"})
			torrent_id_a = torrent_details.find("a", string="Скачать")
			torrent_id = get_torrent_id(torrent_id_a.parent)
			return {
				"id": int(torrent_id),
				"post_id": int(content_table["id"].replace("post_", "")),
				"html_code": torrent_page,
				"magnet_link": magnet_link["href"]
			}
		else:
			return None
		


from datetime import datetime
from typing import List

from bs4 import BeautifulSoup, Tag

from torrent_tracker_search.common.TrackerSearchNotFoundException import TrackerSearchNotFoundException
from torrent_tracker_search.source.nnmclub.model.NNMClubAuthorLink import NNMClubAuthorLink
from torrent_tracker_search.source.nnmclub.model.NNMClubForumLink import NNMClubForumLink
from torrent_tracker_search.source.nnmclub.model.NNMClubSearchItem import NNMClubSearchItem
from torrent_tracker_search.source.nnmclub.model.NNMClubTopicLink import NNMClubTopicLink
from torrent_tracker_search.source.nnmclub.parser import get_forum_id, get_user_id, get_torrent_id, get_topic_id


class NNMClubSearchParser:
	
	@classmethod
	def parse(cls, html: str) -> List[NNMClubSearchItem]:
		if html.count("Не найдено") > 0:
			raise TrackerSearchNotFoundException()
		soup = BeautifulSoup(html, "html.parser")
		content_table = soup.find("table", attrs={"class": "forumline tablesorter"})
		result = list()
		for item_tr in content_table.tbody.childGenerator():
			if type(item_tr) is Tag:
				result.append(cls.parse_item(item_tr))
		return result
		
	@classmethod
	def parse_item(cls, item_tr: Tag) -> NNMClubSearchItem:
		items = [td for td in item_tr.find_all("td")]
		if len(items) == 10:
			_, forum_td, topic_td, author_td, torrent_td, size_td, seed_td, leech_td, reply_td, added_at_td = items
		elif len(items) == 11:
			_, forum_td, topic_td, author_td, torrent_td, size_td, _, seed_td, leech_td, reply_td, added_at_td = items
		else:
			raise Exception(f"Allowed only 10 items. Parser is broken. Now items count {len(items)}")

		forum = NNMClubForumLink(
			id=get_forum_id(forum_td),
			name=str(forum_td.text).strip("\n")
		)
		author = NNMClubAuthorLink(
			id=get_user_id(author_td),
			name=str(author_td.text).strip("\n")
		)
		return NNMClubSearchItem(
			id=get_torrent_id(torrent_td),
			topic=NNMClubTopicLink(
				id=get_topic_id(topic_td),
				name=topic_td.a.text,
			),
			author=author,
			forum=forum,
			size=int(size_td.u.text),
			seed_count=int(seed_td.text),
			leech_count=int(leech_td.text),
			added_at=datetime.fromtimestamp(int(added_at_td.u.text))
		)



from urllib.parse import urlparse

from bs4 import Tag


def get_query_param(source: Tag, param_key: str) -> str:
	source_href = source.a["href"]
	query = urlparse(source_href).query.split("&")
	for item in query:
		if item.startswith(f"{param_key}="):
			query_name, query_value = item.split("=")
			return query_value
	raise Exception(f"Error on get quert param from {source}")

def get_forum_id(forum_td: Tag) -> int:
	return int(get_query_param(forum_td,"f"))

def get_user_id(author_td: Tag) -> int:
	return int(get_query_param(author_td, "pid"))

def get_torrent_id(torrent_td: Tag) -> int:
	return int(get_query_param(torrent_td, "id"))

def get_topic_id(topic_td: Tag) -> int:
	return int(get_query_param(topic_td, "t"))



from dataclasses import dataclass
from datetime import datetime

from torrent_tracker_search.common.model.ISearchResultItem import ISearchResultItem
from torrent_tracker_search.source.nnmclub.model.NNMClubAuthorLink import NNMClubAuthorLink
from torrent_tracker_search.source.nnmclub.model.NNMClubForumLink import NNMClubForumLink
from torrent_tracker_search.source.nnmclub.model.NNMClubTopicLink import NNMClubTopicLink


@dataclass
class NNMClubSearchItem(ISearchResultItem):
	id: int  # id for download torrent file
	author: NNMClubAuthorLink
	forum: NNMClubForumLink
	topic: NNMClubTopicLink
	size: int
	seed_count: int
	leech_count: int
	added_at: datetime
	
	@staticmethod
	def empty(id: int):
		return NNMClubSearchItem(
			id=id,
			author=NNMClubAuthorLink(0, ""),
			forum=NNMClubForumLink(0, ""),
			topic=NNMClubTopicLink(0, ""),
			size=0,
			seed_count=0,
			leech_count=0,
			added_at=datetime.now()
		)
	


from typing import Optional

from bs4 import BeautifulSoup


class NNMClubBBCodeParser:
	
	@classmethod
	def parse(cls, html: str) -> Optional[str]:
		soup = BeautifulSoup(html, "html.parser")
		textarea = soup.find("textarea")
		if textarea is not None:
			return textarea.text
		else:
			return None



from typing import Optional, List

from torrent_tracker_search.common.model.ITorrentFile import ITorrentFile


class ITorrentItem:
	id: int
	post_id: int
	html_code: str
	magnet_link: str
	bb_code: Optional[str] = None
	files: Optional[List[ITorrentFile]] = None



from enum import Enum

from torrent_tracker_search.source.rutracker.RuTrackerClient import RuTrackerClient


class RuTrackerURL(Enum):
	MAIN = "index.php"
	LOGIN = "login.php"
	TRACKER = "tracker.php"
	VIEW_TOPIC = "viewtopic.php"
	VIEW_TORRENT = "viewtorrent.php"
	AJAX = "ajax.php"
	
	@property
	def url(self) -> str:
		return f"{RuTrackerClient.BASE_URL}/{self.value}"



from dataclasses import dataclass

from torrent_tracker_search.common.model.ITorrentFile import ITorrentFile


@dataclass
class NNMClubTorrentFile(ITorrentFile):
	name: str
	is_folder: bool
	files: list
	size: int
	


from dataclasses import dataclass
from typing import Dict

from multidict import CIMultiDict


@dataclass
class TrackerResponse:
	headers: CIMultiDict[str]
	status_code: int
	body: bytes
	
	@property
	def cookies(self) -> Dict[str, str]:
		cookies = {}
		cookie_string = getattr(self.headers, "cookies", "")
		if cookie_string == "":
			return cookies
		cookie_strings = cookie_string.split(";")
		for cookie_string in cookie_strings:
			cookie_name, cookie_value = cookie_string.split("=")
			cookie_name = cookie_name.strip()
			cookies[cookie_name] = cookie_value
		return cookies
	


from dataclasses import dataclass
from typing import Optional, List

from torrent_tracker_search.common.model.ITorrentItem import ITorrentItem
from torrent_tracker_search.source.rutracker.model.RuTrackerTorrentFile import RuTrackerTorrentFile


@dataclass
class RuTrackerTorrentItem(ITorrentItem):
	id: int
	post_id: int
	html_code: str
	magnet_link: str
	bb_code: Optional[str] = None
	files: Optional[List[RuTrackerTorrentFile]] = None



from datetime import datetime
from typing import List

from bs4 import BeautifulSoup, Tag

from torrent_tracker_search.common.TrackerSearchNotFoundException import TrackerSearchNotFoundException
from torrent_tracker_search.source.rutracker.model.RuTrackerAuthorLink import RuTrackerAuthorLink
from torrent_tracker_search.source.rutracker.model.RuTrackerForumLink import RuTrackerForumLink
from torrent_tracker_search.source.rutracker.model.RuTrackerSearchItem import RuTrackerSearchItem
from torrent_tracker_search.source.rutracker.model.RuTrackerTopicLink import RuTrackerTopicLink
from torrent_tracker_search.source.rutracker.parser import get_forum_id, get_user_id, get_topic_id


class RuTrackerSearchParser:
	
	@staticmethod
	def items(html: str) -> List[RuTrackerSearchItem]:
		if html.count("Не найдено") > 0:
			raise TrackerSearchNotFoundException()
		soup = BeautifulSoup(html, "html.parser")
		content_table = soup.find("table", attrs={"id": "tor-tbl"})
		table_body = content_table.tbody
		result = []
		for item in table_body.childGenerator():
			if type(item) is Tag:
				result.append(RuTrackerSearchParser.item(item))
		return result
		
	@staticmethod
	def item(item: Tag) -> RuTrackerSearchItem:
		items = [td for td in item.find_all("td")]
		if len(items) == 10:
			id_td, is_verified_td, forum_td, torrent_td, author_td, size_td, seed_td, leech_td, download_td, added_at_td = items
			forum = RuTrackerForumLink(
				id=get_forum_id(forum_td),
				name=str(forum_td.text).strip("\n")
			)
			author = RuTrackerAuthorLink(
				id=get_user_id(author_td),
				name=str(author_td.text).strip("\n")
			)
			torrent_div, tags_div = torrent_td.find_all("div")
			return RuTrackerSearchItem(
				id=int(id_td["id"]),
				is_verified=is_verified_td.text == "√",
				topic=RuTrackerTopicLink(
					id=get_topic_id(torrent_div),
					name=str(torrent_div.text).strip("\n")
				),
				tags=[tag_span.text for tag_span in tags_div.childGenerator()],
				forum=forum,
				author=author,
				size=int(size_td["data-ts_text"]),
				seed_count=int(seed_td.text),
				leech_count=int(leech_td.text),
				download_count=int(download_td.text),
				added_at=datetime.fromtimestamp(int(added_at_td["data-ts_text"]))
			)
		else:
			raise Exception(f"Allowed only 10 items. Parser is broken. Now items count {len(items)}")
	


from dataclasses import dataclass
from typing import Optional, List

from torrent_tracker_search.common.model.ITorrentItem import ITorrentItem
from torrent_tracker_search.source.nnmclub.model.NNMClubTorrentFile import NNMClubTorrentFile


@dataclass
class NNMClubTorrentItem(ITorrentItem):
	id: int
	post_id: int
	html_code: str
	magnet_link: str
	bb_code: Optional[str] = None
	files: Optional[List[NNMClubTorrentFile]] = None



from typing import List, Optional

from torrent_tracker_search.common.TorrentTracker import TorrentTracker
from torrent_tracker_search.common.model.ISearchResultItem import ISearchResultItem
from torrent_tracker_search.common.model.ITorrentItem import ITorrentItem
from torrent_tracker_search.source.nnmclub.NNMClubService import NNMClubService
from torrent_tracker_search.source.nnmclub.model.NNMClubSearchItem import NNMClubSearchItem
from torrent_tracker_search.source.rutracker.RuTrackerService import RuTrackerService
from torrent_tracker_search.source.rutracker.model.RuTrackerSearchItem import RuTrackerSearchItem


class TorrentTrackerSearch:
	
	def __init__(self, trackers: List[TorrentTracker]):
		self._trackers = trackers
	
	async def search(self, query: str) -> List[ISearchResultItem]:
		search_result = []
		for tracker in self._trackers:
			tracker_result = await tracker.search(query)
			for result in tracker_result:
				search_result.append(result)
		return search_result
		
	async def torrent(self, search_item: ISearchResultItem) -> Optional[ITorrentItem]:
		if type(search_item) is RuTrackerSearchItem:
			for tracker in self._trackers:
				if type(tracker) is RuTrackerService:
					return await tracker.torrent(search_item.id)
		if type(search_item) is NNMClubSearchItem:
			for tracker in self._trackers:
				if type(tracker) is NNMClubService:
					return await tracker.torrent(search_item.id)
		return None
	


from typing import List

from bs4 import BeautifulSoup

from torrent_tracker_search.source.rutracker.model.RuTrackerTorrentFile import RuTrackerTorrentFile


class RuTrackerTorrentFilesParser:
	
	@staticmethod
	def items(html: str) -> List[RuTrackerTorrentFile]:
		soup = BeautifulSoup(html, "html.parser")
		return RuTrackerTorrentFilesParser.parse_tree(soup.ul)
		
	@staticmethod
	def parse_tree(element):
		# Здесь мы собираем информацию обо всех элементах
		data = []
		if element.name == 'li':
			# sub_data = {}
			div = element.find('div')
			if div:
				# Получаем название
				name = div.find('b').text
				# sub_data['name'] = name
				# Получаем возможный идентификатор
				size = div.find('i')
				if size:
					size = int(size.text)
				else:
					size = 0
				
				data.append(RuTrackerTorrentFile(
					name=name,
					size=size,
					files=[],
					is_folder=False
				))
			
			# Обработка вложенного списка
			nested_ul = element.find('ul')
			if nested_ul:
				nested_data = RuTrackerTorrentFilesParser.parse_tree(nested_ul)
				if nested_data:
					new_size = data[-1].size
					for file in nested_data:
						new_size += file.size
					data[-1].is_folder = True
					data[-1].size = new_size
					data[-1].files = nested_data  # Добавляем дочерние элементы
		
		elif element.name == 'ul':
			for li in element.find_all('li', recursive=False):
				data.extend(RuTrackerTorrentFilesParser.parse_tree(li))
		
		return data
	


import os
from typing import List, Optional

from bs4 import BeautifulSoup, Tag

from torrent_tracker_search.source.nnmclub.model.NNMClubTorrentFile import NNMClubTorrentFile


class NNMClubTorrentFilesParser:
	
	@classmethod
	def parse(cls, html: str) -> Optional[NNMClubTorrentFile]:
		soup = BeautifulSoup(html, "html.parser")
		items: List[NNMClubTorrentFile] = []
		content_table = soup.find("table")
		if content_table is None:
			return None
		for item in content_table.tbody.childGenerator():
			if type(item) is Tag:
				cls.parse_item(item, items)
		root_folder = items[0]
		f = cls.reassemble_to_folders(items[1:])
		root_folder.files = f.files
		cls.calculate_folder_size(root_folder)
		return root_folder
	
	@classmethod
	def calculate_folder_size(cls, folder: NNMClubTorrentFile):
		"""Calculate the total size of the folder by summing up the sizes of contained files and subfolders."""
		if not folder.is_folder:
			return folder.size
		
		total_size = 0
		for file in folder.files:
			if file.is_folder:
				total_size += cls.calculate_folder_size(file)
			else:
				total_size += file.size
		folder.size = total_size
		return folder.size
	
	@classmethod
	def reassemble_to_folders(cls, file_list):
		def add_file_to_folder(folder, path_parts, size):
			if not path_parts:
				return
			
			current_part = path_parts[0]
			remaining_parts = path_parts[1:]
			folder_dict = {file.name: file for file in folder.files}
			
			if current_part not in folder_dict:
				# If this part of the path is not in the folder, add it
				if remaining_parts:
					# If there are remaining parts, it's a folder
					new_folder = NNMClubTorrentFile(name=current_part, is_folder=True, files=[], size=0)
					folder.files.append(new_folder)
					add_file_to_folder(new_folder, remaining_parts, size)
				else:
					# If there are no remaining parts, it's a file
					new_file = NNMClubTorrentFile(name=current_part, is_folder=False, size=size, files=[])
					folder.files.append(new_file)
			else:
				# If the folder exists, continue adding the remaining path
				add_file_to_folder(folder_dict[current_part], remaining_parts, size)
		
		root_folder = NNMClubTorrentFile(name='root', is_folder=True, files=[], size=0)
		
		for file in file_list:
			path_parts = file.name.split(os.sep)
			add_file_to_folder(root_folder, path_parts, file.size)
		
		return root_folder
	
	@classmethod
	def parse_item(cls, tr_code: Tag, items: list):
		number, name, size_in_bytes, size_in_bites = list(tr_code.childGenerator())
		if number.text.strip() == "Папка":
			items.append(
				NNMClubTorrentFile(
					name=name.text.strip(),
					is_folder=True,
					files=[],
					size=0
				)
			)
		else:
			items.append(
				NNMClubTorrentFile(
					name=name.text.strip(),
					is_folder=False,
					files=[],
					size=int(size_in_bites.text.replace(",", ""))
				)
			)
		


import base64
import json
from typing import Optional, List

from torrent_tracker_search.common.TorrentTracker import TorrentTracker
from torrent_tracker_search.common.TrackerClient import TrackerClient
from torrent_tracker_search.source.rutracker.RuTrackerClient import RuTrackerClient
from torrent_tracker_search.source.rutracker.model.RuTrackerSearchItem import RuTrackerSearchItem
from torrent_tracker_search.source.rutracker.model.RuTrackerTorrentFile import RuTrackerTorrentFile
from torrent_tracker_search.source.rutracker.model.RuTrackerTorrentItem import RuTrackerTorrentItem
from torrent_tracker_search.source.rutracker.parser.RuTrackerSearchParser import RuTrackerSearchParser
from torrent_tracker_search.source.rutracker.RuTrackerURL import RuTrackerURL
from torrent_tracker_search.source.rutracker.parser.RuTrackerTorrentFilesParser import RuTrackerTorrentFilesParser
from torrent_tracker_search.source.rutracker.parser.RuTrackerTorrentParser import RuTrackerTorrentParser


class RuTrackerService(TorrentTracker):
	
	def __init__(self, client: TrackerClient):
		self._client: TrackerClient = client
	
	async def search(self, query: str) -> List[RuTrackerSearchItem]:
		response = await self._client.get(
			url=RuTrackerURL.TRACKER.url,
			params={
				"nm": query
			}
		)
		body = response.body.decode("Windows-1251")
		return RuTrackerSearchParser.items(body)
	
	async def torrent(self, torrent_id: int) -> Optional[RuTrackerTorrentItem]:
		response = await self._client.get(
			url=RuTrackerURL.VIEW_TOPIC.url,
			params={
				"t": torrent_id
			}
		)
		body = response.body.decode("Windows-1251")
		torrent_dict = RuTrackerTorrentParser.parse(body)
		if torrent_dict is not None:
			form_token = RuTrackerTorrentParser.form_token(body)
			bb_code = await self.torrent_bb_code(post_id=torrent_dict["post_id"], form_token=form_token)
			if bb_code is not None:
				bb_code = base64.standard_b64encode(bb_code.encode()).decode()
			return RuTrackerTorrentItem(
				id=torrent_dict["torrent_id"],
				post_id=torrent_dict["post_id"],
				html_code=base64.standard_b64encode(torrent_dict["html_code"]).decode(),
				magnet_link=torrent_dict["magnet_link"],
				bb_code=bb_code,
				files=await self.torrent_files(torrent_id=torrent_id)
			)
		
	async def torrent_files(self, torrent_id: int) -> List[RuTrackerTorrentFile]:
		response = await self._client.post_form_data(
			url=RuTrackerURL.VIEW_TORRENT.url,
			body={
				"t": torrent_id
			}
		)
		body = response.body.decode("Windows-1251")
		return RuTrackerTorrentFilesParser.items(body)
	
	async def torrent_bb_code(self, post_id: int, form_token: str) -> Optional[str]:
		response = await self._client.post_form_data(
			url=RuTrackerURL.AJAX.url,
			body={
				"action": "view_post",
				"post_id": post_id,
				"mode": "text",
				"form_token": form_token
			}
		)
		body = response.body.decode()
		body_json: dict = json.loads(body)
		return body_json["post_text"]
		
	async def login_by_credentials(self, login: str, password: str) -> Optional[str]:
		response = await self._client.post_form_data_encoded(
			url=RuTrackerURL.LOGIN.url,
			body={
				"login_username": login,
				"login_password": password,
				"login": ""
			},
			allow_redirects=False
		)
		new_cookies = response.headers.getall("Set-Cookie")
		for new_cookie in new_cookies:
			if new_cookie.startswith(RuTrackerClient.SESSION_COOKIE_KEY):
				cookie_string = new_cookie.split(";")[0]
				cookie_name, cookie_value = cookie_string.split("=")
				self._client.session.cookie_jar.update_cookies({RuTrackerClient.SESSION_COOKIE_KEY: cookie_value})
				return cookie_value
		return None
	
	def login_by_session(self, session_id: str):
		self._client.session.cookie_jar.update_cookies({RuTrackerClient.SESSION_COOKIE_KEY: session_id})



from datetime import datetime


class Link:
	id: int
	name: str
	

class ISearchResultItem:
	id: int  # id for download torrent file
	author: Link
	forum: Link
	topic: Link
	size: int
	seed_count: int
	leech_count: int
	added_at: datetime
	


from typing import Optional, List

from torrent_tracker_search.common.model.ISearchResultItem import ISearchResultItem
from torrent_tracker_search.common.model.ITorrentItem import ITorrentItem


class TorrentTracker:
	
	async def search(self, query: str) -> List[ISearchResultItem]: ...
	
	async def torrent(self, topic_id: int) -> Optional[ITorrentItem]: ...



from typing import List, Optional

from torrent_tracker_search.common.TorrentTracker import TorrentTracker
from torrent_tracker_search.common.TrackerClient import TrackerClient
from torrent_tracker_search.source.nnmclub.NNMClubClient import NNMClubClient
from torrent_tracker_search.source.nnmclub.NNMClubURL import NNMClubURL
from torrent_tracker_search.source.nnmclub.model.NNMClubSearchItem import NNMClubSearchItem
from torrent_tracker_search.source.nnmclub.model.NNMClubTorrentFile import NNMClubTorrentFile
from torrent_tracker_search.source.nnmclub.model.NNMClubTorrentItem import NNMClubTorrentItem
from torrent_tracker_search.source.nnmclub.parser.NNMClubBBCodeParser import NNMClubBBCodeParser
from torrent_tracker_search.source.nnmclub.parser.NNMClubSearchParser import NNMClubSearchParser
from torrent_tracker_search.source.nnmclub.parser.NNMClubTorrentFilesParser import NNMClubTorrentFilesParser
from torrent_tracker_search.source.nnmclub.parser.NNMClubTorrentParser import NNMClubTorrentParser


class NNMClubService(TorrentTracker):
	
	def __init__(self, client: TrackerClient):
		self._client: TrackerClient = client
	
	async def search(self, query: str) -> List[NNMClubSearchItem]:
		response = await self._client.get(
			url=NNMClubURL.TRACKER.url,
			params={
				"nm": query,
				"f": "-1",
				"search_submit": ""
			}
		)
		body = response.body.decode("Windows-1251")
		return NNMClubSearchParser.parse(body)
		
	def login(self, session_id: str):
		self._client.session.cookie_jar.update_cookies({NNMClubClient.SESSION_COOKIE_KEY: session_id})
	
	async def torrent_files(self, torrent_id: int) -> Optional[NNMClubTorrentFile]:
		response = await self._client.get(
			url=NNMClubURL.FILE_LIST.url,
			params={
				"attach_id": torrent_id
			}
		)
		return NNMClubTorrentFilesParser.parse(response.body.decode("Windows-1251"))
	
	async def torrent_bb_code(self, torrent_post_id: int) -> Optional[str]:
		response = await self._client.get(
			url=NNMClubURL.POSTING.url,
			params={
				"mode": "quote",
				"p": torrent_post_id
			}
		)
		return NNMClubBBCodeParser.parse(response.body.decode("Windows-1251"))
	
	async def torrent(self, topic_id: int) -> Optional[NNMClubTorrentItem]:
		response = await self._client.get(
			url=NNMClubURL.VIEW_TOPIC.url,
			params={
				"t": topic_id
			}
		)
		data = NNMClubTorrentParser.parse(response.body.decode("Windows-1251"))
		if data is not None:
			return NNMClubTorrentItem(
				id=data["id"],
				post_id=data["post_id"],
				html_code=data["html_code"],
				magnet_link=data["magnet_link"],
				bb_code=await self.torrent_bb_code(torrent_post_id=data["post_id"]),
				files=await self.torrent_files(torrent_id=data["id"])
			)
		else:
			return None



import json
from typing import Optional

from bs4 import BeautifulSoup


class RuTrackerTorrentParser:
	
	@staticmethod
	def form_token(html: str) -> str:
		form_token_start_index = html.index("form_token")
		form_token_end_index = html[form_token_start_index:].index(",")
		raw_form_token = html[form_token_start_index:form_token_start_index+form_token_end_index]
		form_token = raw_form_token.strip().replace("form_token: ", "").replace("'", "")
		return form_token
	
	@staticmethod
	def parse(html: str) -> Optional[dict]:
		soup = BeautifulSoup(html, "html.parser")
		torrent_html = soup.find("div", attrs={"class": "post_body"})
		if torrent_html is not None:
			props = json.loads(torrent_html["data-ext_link_data"])
			post_id = int(props["p"])
			torrent_id = int(props["t"])
			magnet_link_a = soup.find("a", attrs={"data-topic_id": torrent_id})
			magnet_link = magnet_link_a["href"]
			return {
				"post_id": post_id,
				"torrent_id": torrent_id,
				"magnet_link": magnet_link,
				"html_code": torrent_html.encode()
			}



from typing import Optional

from aiohttp import ClientSession

from torrent_tracker_search.common.TrackerResponse import TrackerResponse


class TrackerClient:
	
	session: Optional[ClientSession] = None
	
	async def get(self, url: str, params: Optional[dict] = None, allow_redirects: bool = True) -> TrackerResponse: ...
	
	async def post_raw(self, url: str, body: dict, params: Optional[dict] = None, allow_redirects: bool = True) -> TrackerResponse: ...
	
	async def post_form_data(self, url: str, body: dict, params: Optional[dict] = None, allow_redirects: bool = True) -> TrackerResponse: ...
	
	async def post_form_data_encoded(self, url: str, body: dict, params: Optional[dict] = None, allow_redirects: bool = True) -> TrackerResponse: ...
	


from urllib.parse import urlparse

from bs4 import Tag


def get_forum_id(forum_td: Tag) -> int:
	forum_href = forum_td.div.a["href"]
	query = urlparse(forum_href).query.split("&")
	for item in query:
		if item.startswith("f="):
			query_name, query_value = item.split("=")
			return int(query_value)
	raise Exception(f"Error on get forum id from {forum_href}")

def get_user_id(author_td: Tag) -> int:
	author_href = author_td.div.a["href"]
	query = urlparse(author_href).query.split("&")
	for item in query:
		if item.startswith("pid="):
			query_name, query_value = item.split("=")
			return int(query_value)
	raise Exception(f"Error on get author id from {author_td}")

def get_topic_id(topic_td: Tag) -> int:
	topic_id = topic_td.a.get("data-topic_id", None)
	if topic_id is not None:
		return int(topic_id)
	else:
		raise Exception(f"Error on get topic id from {topic_td}")

